module.exports = function (grunt) {

	var srcFiles = {
		js: ["src/assets/js/controllers/**/*.js"],
		less: ["src/assets/css/**/*.less"],
		html: ["src/templates/**/*.html"]
	};

	grunt.initConfig({
		jshint: {
			all: srcFiles.js.concat(["GruntFile.js"]),
			options: {
				evil: true
			}
		},
		watch: {
			js: {
				files: srcFiles.js,
				tasks: ['jshint', 'concat:js']
			},
			html: {
				files: srcFiles.html,
				tasks: ['copy']
			}
		},
		concat: {
			js: {
				options: {
					process: function (src, filepath) {
						return '// FILE: ' + filepath + '\n' + src + '\n';
					}
				},
				src: srcFiles.js,
				dest: 'dist/rui-fields.js'
			}
		},
		uglify: {
			options: {
				mangle: false,
				compress: false,
				report: 'min'
			},
			dist: {
				files: {
					'dist/rui-fields.min.js': srcFiles.js
				}
			}
		},
		copy: {
			main: {
				files: [
					{
						expand: true,
						cwd: 'src/',
						src: 'templates/**/*',
						dest: 'dist/'
					}
				]
			}
		},
		bump: {
			options: {
				files: ['bower.json', 'package.json'],
				pushTo: 'origin',
				commitFiles: ["-a"],
				push: true
			}
		},
		less: {
			development: {
				options: {
					compress: true,
					modifyVars: {
						fontFamily:'Roboto, san serif'
					}
				},
				files: {
					'dist/rui-fields.css': srcFiles.less
				}
			}
		}
	});

	grunt.loadNpmTasks('grunt-contrib-less');
	grunt.loadNpmTasks('grunt-contrib-jshint');
	grunt.loadNpmTasks('grunt-contrib-watch');
	grunt.loadNpmTasks('grunt-contrib-concat');
	grunt.loadNpmTasks('grunt-contrib-copy');
	grunt.loadNpmTasks('grunt-contrib-uglify');
	grunt.loadNpmTasks('grunt-bump');

	grunt.registerTask('default', ['jshint', 'concat', 'uglify', 'copy', 'less']);

};