function PatientDetailPopupController(ApplicationContext, templateManager, ScreenService, eventAggregator, ZHPromise) {
    var self = this;
    
    var patientsRepository = ApplicationContext.PatientsRepository;
    var bedsService = ApplicationContext.BedsService;
    
    this.openPopup = function(detailPopupWidget, patient){
        detailPopupWidget.isVisible = true;
        detailPopupWidget.isEditingPatient = false;
        detailPopupWidget.patient = patient;
        
        if (patient.bed) {
            bedsService.getBed(patient.bed).then(function(bedObject) {
                detailPopupWidget.bed = bedObject;
                templateManager.updateTemplate();
            });
        } else {
            detailPopupWidget.bed = {
                nullBed: true
            };
        }
    };
    this.closePopup = function(detailPopupWidget){
        if (!detailPopupWidget.isEditingPatient) {
            detailPopupWidget.isVisible = false;
            delete detailPopupWidget.patient;
        }
    };
    this.editPatientDetails = function(detailPopupWidget) {
        detailPopupWidget.backupRow = {};
        detailPopupWidget.backupRow.patient = Object.assign({}, detailPopupWidget.patient);
        detailPopupWidget.backupRow.bed = Object.assign({}, detailPopupWidget.bed);
        detailPopupWidget.choosedOptions = [];
        bedsService.getVacantBeds().then(function(beds) {
            let vacantBeds = beds || [];
            let currentBedArr = [detailPopupWidget.bed];
            detailPopupWidget.vacantBeds = currentBedArr.concat(vacantBeds);

            detailPopupWidget.isEditingPatient = true;
        });
        var bed = detailPopupWidget.getField('bedName');
        bed.dataSourceFilter = [];
        detailPopupWidget.currentRow['storeyName'] = '';
        detailPopupWidget.currentRow['hallName'] = '';
        detailPopupWidget.currentRow['bedName'] = '';
        eventAggregator.publish("reloadAllFields", detailPopupWidget);
    };
    function convertStringToJson(string){
        try{
            return JSON.parse(string);
        }catch (e){
            return false;
        }
    }
    this.saveEditPatientDetails = function(detailPopupWidget) {
        
       let patientToSave = detailPopupWidget.patient;
        patientToSave.changed = true;
        var bed = convertStringToJson(detailPopupWidget.currentRow['bedName']);
        if(bed){
            patientToSave.bed = bed.bed;
            patientToSave.bedName = bed.bedName;
            patientToSave.hall = bed.hall;
            patientToSave.storey = bed.storey;
            patientToSave.oldBed = detailPopupWidget.backupRow.bed;
            patientsRepository.getPatientById(patientToSave.id).then(function(patient) {
                patient = Object.assign(patient, patientToSave);
                var promise = ZHPromise.defer();
                var params = {
                    'dataSourceName': '/Patients',
                    'rowToSave': [patient],
                    'route': '/Patients/save',
                    'type' : 'DataSet',
                    'promise': promise
                };
                eventAggregator.publish('saveAlterPatient', params);
                promise.then(function(){
                    patientsRepository.save(patient).then(function(){
                        bedsService.getBed(bed.bed).then(function(vacantBedToOccupy){
                            bedsService.getBed(detailPopupWidget.backupRow.bed.bed).then(function(occupiedBedToVacate){
                                bedsService.localSwapBeds(vacantBedToOccupy, occupiedBedToVacate);
                                detailPopupWidget.bed = bed;
                                detailPopupWidget.isEditingPatient = false;
                                changeDataSource(detailPopupWidget.getField('hallName'), '/HallsDistincts', true);
                                delete detailPopupWidget.backupRow;
                            });
                        });
                    });
                });
            });
        }else{
            ScreenService.showMessage(ScreenService.i18n('Selecione um leito!'));
        }
    };
    function changeDataSource(field, dataSourceName, clearFilters){
        field.dataSource.name = dataSourceName;
        if(clearFilters){
            field.dataSourceFilter = [];
        }
        field.reload();
    }
    this.cancelEditPatientDetails = function(detailPopupWidget) {
        var fieldHall = detailPopupWidget.getField('hallName');
        var fieldBed = detailPopupWidget.getField('bedName');
        detailPopupWidget.isEditingPatient = false;
        
        detailPopupWidget.patient = Object.assign(detailPopupWidget.patient, detailPopupWidget.backupRow.patient);
        detailPopupWidget.bed = Object.assign(detailPopupWidget.bed, detailPopupWidget.backupRow.bed);
        changeDataSource(fieldHall, '/HallsDistincts', true);
        fieldBed.dataSourceFilter = [];
        fieldBed.reload();
        delete detailPopupWidget.backupRow;
    };
    this.changeStorey = function(widget){
        var value = JSON.parse(widget.currentRow['storeyName']);
        widget.getField('bedName').dataSourceFilter = [{
            "name": "storey",
            "operator": "=",
            "value": value.storey
        }];
        widget.getField('hallName').dataSource.name = '/Halls';
        widget.getField('hallName').dataSourceFilter = [{
            "name": "storey",
            "operator": "=",
            "value": value.storey
        }];
        widget.currentRow['bedName'] = '';
        widget.getField('bedName').reload();
        widget.currentRow['hallName'] = '';
        widget.getField('hallName').reload();
    };
    this.changeBed = function(widget){
        var valor = JSON.parse(widget.currentRow['bedName']);
        widget.currentRow['storeyName'] = valor.storey;
        widget.currentRow['hallName']   = valor.hall;
    };
    this.changeHall = function(widget){
        var bed = widget.getField('bedName');
        if(!hasHallInFilter(bed)){
            bed.dataSourceFilter.push({
                "name": "hall",
                "operator": "=",
                "value": widget.currentRow['hallName']
            });
        }else{
            bed.dataSourceFilter.forEach(function(filtro){
               if(filtro.name === "hall"){
                   filtro.operator = "=";
                   filtro.value = widget.currentRow['hallName'];
               }
            });
        }
        var storey = convertStringToJson(widget.currentRow['storeyName']);
        if(typeof storey === "number"){
            bed.dataSourceFilter.push({
                "name": "storey",
                "operator": "=",
                "value": storey
            });
        }
        widget.currentRow['bedName'] = '';
        bed.reload();
    };
    
    function hasHallInFilter(field){
        var has = false;
        if(field.dataSourceFilter){
            field.dataSourceFilter.forEach(function(filtro){
               if(filtro.name === "hall"){
                   has = true;
               } 
            });
        }
        return has;
    }
    
    this.changePatientSituation = function(detailPopupWidget, patient, situation) {
        let messages = [
            'Deseja confirmar <b>alta</b> para '+patient.formattedName+'?',
            'Deseja confirmar <b>óbito</b> para '+patient.formattedName+'?'
        ];
        ScreenService.confirmMessage(messages[situation], 'question', function() {
            eventAggregator.publish("sendPatientUpdate", {'patient': [patient], 'situation': situation,'widget': detailPopupWidget});
            self.closePopup(detailPopupWidget);
        });
    };
};