function PatientDetailsController($scope, $timeout, eventAggregator, ZHPromise, localStorageEngine, ScreenService, ApplicationContext, templateManager, $element) {
    /*global moment, angular, $, Util, excelFormulaUtilities, getLocalVar*/
    var widget = $scope.widget;
    var controlFeedingCrud = ApplicationContext.CRUD.init("/ControlFeedings");
    var lactaryFeedingCrud = ApplicationContext.CRUD.init("/LactaryFeedings");
    var enteralFeedingCrud = ApplicationContext.CRUD.init("/EnteralFeedings");
    var normalFeedingCrud = ApplicationContext.CRUD.init("/NormalFeedings");

    $scope.bindScrollListener = function(selector) {
        if(!selector){
            selector = '.list-plates';
        }
        var element = $element.get(0);
        var listPlates = element.querySelector(selector);
        listPlates.addEventListener('scroll', function(e) {
            var scrollHeight = e.target.scrollHeight - 15;
            var scrollPosition = e.target.clientHeight + e.target.scrollTop - 15; //padding
            var previousScroll = previousScroll || scrollPosition;
            if (((scrollHeight - scrollPosition) / scrollHeight < 0.030) && previousScroll <= scrollPosition) {
                loadData(true);
            }
            previousScroll = scrollPosition;
        });
    };
    
    function loadData(apply) {
        if (apply) {
            $scope.$apply();
        }
        var widget = $scope.widget;
        nextPage(widget).then(function(callback) {
        	var data;
            if (callback && callback.dataset) {
                Object.keys(callback.dataset).forEach(function (datasetName) {
                    data = callback.dataset[datasetName];
                });
            } else {
                data = callback || [];
            }
            $scope.loadedAllData = data.length < widget.itemsPerPage;
            $scope.loadingData = false;
        }.bind(this));
    }
    
    function nextPage (widget) {
        if (widget.dataSource) {
            return widget.dataSource.load(widget.getDataSourceFilter(), widget.dataSource.getNextPage(), widget.itemsPerPage);
        } else {
            return ZHPromise.when();
        }
    }
    
    $scope.updateProcedurePrescription = function(widget){
        eventAggregator.publish('updateProcedureRestriction', widget);
    };

    function setClassification(bioAval) {
        if (!bioAval) {
            return null;
        }
        var FORMULA = bioAval.examinationFormula;
        if (FORMULA) {
            var result = FORMULA.replace(/SEXO/g, 'window.SEXO').replace(/IDADE/g, 'window.IDADE').replace(/RESULTADO/g, 'window.RESULTADO');
            window.IDADE = bioAval.age;
            window.RESULTADO = bioAval.examinationValue;
            if (bioAval.gender === 0) {
                window.SEXO = 'MALE';
            } else {
                window.SEXO = 'FEMALE';
            }
            var formulaJS = excelFormulaUtilities.formula2JavaScript(result);
            formulaJS = formulaJS.split("AND").join("&&");
            
            if(formulaJS.charAt(formulaJS.length-1) !== ")"){
                formulaJS = formulaJS.substr(0, formulaJS.length-1);
            }
            try{
                return eval(formulaJS); // jshint ignore:line|
            }catch(error){
                return null;
            }
        } else {
            return null;
        }
    }

    function createNewFeeding(feedingData) {
        widget.newControlData = true;
        widget.filtredFeedingsForWidget = [];
        var idtpservhosp = { //Servico
            'oral': '0',
            'pare': '1',
            'lact': '2'
        };
        var idformdiet = { //Diet
            'oral': ['0', '3', '4'],
            'lact': '1',
            'pare': '2'
        };
        var typeClicked = feedingData;
        feedingData = {};
        feedingData.unit = localStorageEngine.getLocalVar('__CDFILIAL');
        feedingData.idtpservhosp = idtpservhosp[typeClicked];
        feedingData.idformdiet = idformdiet[typeClicked];
        feedingData.id = widget.curPatient.id;
        feedingData.typeClicked = typeClicked;
        feedingData.serviceDate = moment().format("YYYY-MM-DD");
        return feedingData;
    }
    $scope.onClickSelectNative = function(event, field){
        var select = document.activeElement;
        if(!field.addEventBlur){
            document.activeElement.onblur = function(){
                field.readOnly = true;
                templateManager.updateTemplate();
            };
            field.addEventBlur = true;
        }
        if(!templateManager.keyboardOpen){
                field.readOnly = false;
                templateManager.updateTemplate();
        } else {
            if(Util.isPhone() || Util.isTablet()){
                field.readOnly = true;
                templateManager.updateTemplate();
                setTimeout(function(){
                    field.readOnly = false;
                    templateManager.updateTemplate();
                    select.click();
                    event.stopPropagation();
                },800);
            } else {
                field.readOnly = false;
                templateManager.updateTemplate();
            }
        }
    };
    function filterServicesForFeedingControlPopUp(feedingData, filter, isNew) {
        return localStorageEngine.getDataSource('/Services', filter).then(function (filteredServices) {
            widget.filteredServicesFeeding = filteredServices;

            filter = [
                {
                    "name": "unit",
                    "operator": "=",
                    "value": localStorageEngine.getLocalVar('__CDFILIAL')
                }
            ];
            var service = feedingData.cdservfili ? feedingData.cdservfili : feedingData.serviceId; 
            if (!isNew) {
                filter.push({
                    "name": "serviceId",
                    "operator": "=",
                    "value": service
                });
            }

            if (feedingData.idtpservhosp === "0") {//if Oral
                filter.push({
                    "name": "service",
                    "operator": "IN",
                    "value": ['0', '3', '4']
                });
            } else {
                var filterService = feedingData.idformdiet ? feedingData.idformdiet : feedingData.dietService;
                filter.push({
                    "name": "service",
                    "operator": "=",
                    "value": filterService
                });
            }

            filterDietsForFeedingControlPopUp(feedingData, filter, isNew);
        })
            .catch(function (error) {
                console.log(error);
            });
    }

    function filterDietsForFeedingControlPopUp(feedingData, filter, isNew) {
        localStorageEngine.getDataSource('/DietsServices', filter).then(function (filteredDiets) {
            // widget.filteredDietsFeeding = isNew ? [] : filteredDiets;     //Changed for tests
            widget.filteredDietsFeeding = isNew ? filteredDiets : filteredDiets;
            // $timeout(function () {
            openPopupFeedingControl(feedingData, isNew);
            // }, 100);
        });
    }

    function openPopupFeedingControl(feedingData, isNew) {
        var widgetPopup = widget.container.getWidget('widgetPopupPrescricao');
        widgetPopup.currentRow = feedingData.id ? angular.copy(feedingData) : feedingData;
        widgetPopup.currentRow.serviceDate = moment(widgetPopup.currentRow.serviceDate, "DD/MM/YYYY").format("YYYY-MM-DD");
        widgetPopup.currentRow.__is_new = isNew;
        widgetPopup.container.showMenu = false;

        if (isNew) {
            widget.edit();
        } else {
            widgetPopup.currentRow.indexOfOriginalControlFeedings = widget.localDataSourceLoaded ? widget.localDataSourceLoaded.ControlFeedings.indexOf(feedingData) : null;
            widgetPopup.currentRow.indexOfOriginalCurPatFeedings = widget.curPatient ? widget.curPatient.ControlFeedings.indexOf(feedingData) : null;
            
            if (widgetPopup.currentRow.idtpservhosp === '0') { //Oral feeding
                openOralEditor(feedingData, isNew);
            } else if (widgetPopup.currentRow.idtpservhosp === '1') { //Enteral feeding
                openEnteEditor(feedingData, isNew);
            } else { //Lactary Feeding
                openLacEditor(feedingData, isNew);
            }
        }
        widget.container.showMenu = false;
       //widget.currentRow.isEditting = true;
    }

    function openOralEditor(feedingData, isNew) {
        var widgetPopup = widget.container.getWidget('widgetPopupPrescricao');
        var dtCard = moment(feedingData.serviceDate, "DD/MM/YYYY");
            
            dtCard = dtCard.isValid() ? dtCard.format("DD/MM/YYYY") : moment(feedingData.serviceDate, "YYYY-MM-DD")
            .format("DD/MM/YYYY");
        var filter = [{
            "name": "unit",
            "operator": "=",
            "value": localStorageEngine.getLocalVar('__CDFILIAL')
        }, 
        {
            "name": "id",
            "operator": "=",
            "value": localStorageEngine.getLocalVar('__patient')
        },
        {
            "name": "serviceDate",
            "operator": "=",
            "value": dtCard
        },
        {
            "name": "sequential",
            "operator": "=",
            "value": feedingData.sequential
        },
        {
            "name": [{
                "name": "unique",
                "direction": "ASC"
            }],
            "operator": "ORDER_BY",
            "value": ""
        }];
        if (widget.container.name !== 'MapKitchenMaid') {
            var patientIntoRestriction = widget.container.getWidget('WidgetDetailsPatient').curPatient.PatientIntoRestriction;
            if(Array.isArray(patientIntoRestriction) && patientIntoRestriction.length){
                widgetPopup.currentRow.PatientIntoRestriction = patientIntoRestriction.filter(function(intoRestriction){
                    return intoRestriction.NRSEQUSERV === feedingData.sequential && intoRestriction.DTSERVINTE === feedingData.serviceDate;
                });
            } else {
                widgetPopup.currentRow.PatientIntoRestriction = [];
            }
        } else {
            widgetPopup.currentRow.PatientIntoRestriction = [];
        }
        normalFeedingCrud.find(filter,false,'/NormalFeedings').then(function (feedings) {
            feedings.map(function(feeding){
                feeding.plateObservationMode = !!feeding.observation;
                feeding.plateTypeName = feeding.isRequiredOnMenu ? feeding.plateTypeName
                    : feeding.plateTypeName + ' (' + ScreenService.i18n('Opção') + ')';
            });
            var arrayFinal = [];
            feedings.reduce(function(prev, current) { // funções para organizar e separar o primeiro registro de cada tipo de prato
                var index = prev.findIndex(function(j) {// a fim de escrever a label apenas acima do primeiro
                    return j.plateType === current.plateType;
                });
                if (index < 0) {
                    index = prev.length;
                    prev.push({
                        plateType: current.plateType,
                        lista: []
                    });
                }
                prev[index].lista.push(current);
                return prev;
            }, []).map(function(item) {
                return item.lista;
            }).forEach(function(it) {
                if (it.length === 0) {
                    it.isFirst = true;
                    arrayFinal.push(it);
                } else {
                    for (var i = 0; i < it.length; i++) {
                        if (i === 0) {
                            it[i].isFirst = true;
                        }
                        arrayFinal.push(it[i]);
                    }
                }
            });
            widgetPopup.filtredFeedingsForWidget = angular.copy(arrayFinal);
            if (!widgetPopup.currentRow.selectedDiet && widgetPopup.currentRow.dietId) {
                widgetPopup.currentRow.selectedDiet = [];
                widgetPopup.currentRow.selectedDiet.dietId = widgetPopup.currentRow.dietId;
            }
            if(widgetPopup.currentRow.idformdiet === '4' || widgetPopup.currentRow.dietService === '4'){
                widgetPopup.bkpFeedings = angular.copy(arrayFinal);
            }
        });
        widgetPopup.currentRow.serviceId = widgetPopup.currentRow.cdservfili;
        widgetPopup.getField('serviceId').dataSource.data = 
            [{
                "serviceId":widgetPopup.currentRow.serviceId,
                "serviceName":widgetPopup.currentRow.serviceName
            }];
        widgetPopup.getField('dietId').dataSource.data = 
            [{
                "dietId":widgetPopup.currentRow.dietId,
                "dietName":widgetPopup.currentRow.dietName
            }];    
        widgetPopup.getField('dietId').value(widgetPopup.currentRow.dietId);
        ScreenService.openPopup(widgetPopup);
        widgetPopup.edit();
    }
    function openEnteEditor(feedingData, isNew) {
        var widgetPopup = widget.container.getWidget('widgetPopupPrescricao');
        var fieldServico = widgetPopup.getField('serviceId');
        var fieldDieta = widgetPopup.getField('dietId');
        fieldServico.readOnly = true;
        fieldDieta.readOnly = true;
        widgetPopup.currentRow.serviceId = widgetPopup.currentRow.cdservfili;
        widgetPopup.getField('serviceId').dataSource.data = 
            [{
                "serviceId":widgetPopup.currentRow.serviceId,
                "serviceName":widgetPopup.currentRow.serviceName
            }];
        widgetPopup.getField('dietId').dataSource.data = 
            [{
                "dietId":widgetPopup.currentRow.dietId,
                "dietName":widgetPopup.currentRow.dietName
            }];
        widgetPopup.getField('dietId').value(widgetPopup.currentRow.dietId);
        ScreenService.openPopup(widgetPopup);
        widgetPopup.view();
    }
    function openLacEditor(feedingData, isNew) {
        var widgetPopup = widget.container.getWidget('widgetPopupPrescricao');
        widgetPopup.currentRow.dietName = !widgetPopup.currentRow.dietName ? widgetPopup.currentRow.selectedDiet.dietName : widgetPopup.currentRow.dietName;
        widgetPopup.currentRow.serviceId = widgetPopup.currentRow.cdservfili;
        widgetPopup.getField('serviceId').dataSource.data = 
            [{
                "serviceId":widgetPopup.currentRow.serviceId,
                "unit":widgetPopup.currentRow.unit,
                "serviceName":widgetPopup.currentRow.serviceName
            }];
        widgetPopup.getField('dietId').dataSource.data = 
            [{
                "dietId":widgetPopup.currentRow.dietId,
                "dietName":widgetPopup.currentRow.dietName
            }];
        widgetPopup.getField('dietId').value(widgetPopup.currentRow.dietId);
        ScreenService.openPopup(widgetPopup);
        widgetPopup.view();
    }

    function setLactaryFeedingColumnsAndPropertiesOnFieldEdited(controlFeeding){
        
        var lactaryFeedingColumns = ["Data", "Hora", "Formulação", "Qtd.Formulação"];
        var lactaryFeedingProperties = ["lactaryDate", "hourLactaryPlate", "formulationName", "plateNumbers"];
        
        controlFeeding.columns = lactaryFeedingColumns;
        controlFeeding.propertyFeedings = lactaryFeedingProperties;
        
    }
    
    function getCurEnteralFeeding(feedingData) {
        var filter = [
            {
                "name": "serviceDate",
                "operator": "=",
                "value": feedingData.serviceDate
            }, 
            {
                "name": "id",
                "operator": "=",
                "value": widget.curPatient.id
            },
            {
                "name": "sequential",
                "operator": "=",
                "value": feedingData.sequential
            }
        ];
        enteralFeedingCrud.find(filter,false,'/EnteralFeedings').then(function (entFeeding) {
            var info = ['enteralDate', 'time', 'accessRoadName', 'preparationName', 'formulationName', 'volPrescription', 'volInfudido', 'serviceNumbers'];
            var header = ['Data', 'Hora', 'Via de Acesso', 'Preparação', 'Formulação', 'Vol.Prescrito', 'Vol.Infundido', 'Velocidade (ML/H)'];

            //## FOR INPUTS ON ADITION  # d = aberto ou fechado # n = nao tem # f = funcao
            var cds = ['n', 'n', 'accessRoadId', 'n', 'formulationId', 'n', 'n', 'n'];
            var source = ['n', 'n', 'AccessRoads', 'd', 'Formulations', 'n', 'n', 'n'];
            widget.fieldSource = {};

            var fields = [];
            for (var i = 0; i < header.length; i++) {
                header[i] = ScreenService.i18n(header[i]);
                fields.push({
                    'name': header[i],
                    'value': info[i],
                    'source': source[i],
                    'cd': cds[i]
                });
            }

            fields.forEach(function (field) {
                if (field.source !== 'n' &&
                    field.source !== 'f' &&
                    field.source !== 'd') {
                    var source = '/' + field.source;

                    localStorageEngine.getDataSource(source, null).then(function (data) {
                        widget.fieldSource[field.source] = data;
                    });

                }
            });

            fields.push({
                'name': ScreenService.i18n('Observação'),
                'value': 'observation',
                'source': 'n',
                'cd': 'n'
            });
            widget.subItemData = [];
            feedingData.propertyFeedings = info;
            feedingData.fields = fields;
            feedingData.colums = header;
            feedingData.feedings = entFeeding;

        });
    }

    function createOrEditFeedingControl(feedingData) {
        feedingData.serviceDate = feedingData.serviceDate ? moment(feedingData.serviceDate, "DD/MM/YYYY").format("DD/MM/YYYY") : moment().format("DD/MM/YYYY");
        var originalIsNewStatus = feedingData.__is_new ? feedingData.__is_new : false;
        if (feedingData.isNewRegister) {
            feedingData = createNewFeeding(feedingData);
        } else {
            widget.newControlData = false;
        }
        feedingData.originalIsNewStatus = originalIsNewStatus;
        feedingData.isNewRegister = feedingData.hasOwnProperty('isNewRegister') ? feedingData.isNewRegister : true; 
        var service = feedingData.idtpservhosp;
        var filter = [
            {
                "name": "service",
                "operator": "=",
                "value": service
            }, {
                "name": "unit",
                "operator": "=",
                "value": localStorageEngine.getLocalVar('__CDFILIAL')
            }
        ];

        if (feedingData.idtpservhosp === '0' || feedingData.idtpservhosp === '3' || feedingData.idtpservhosp === '4') {//ORAL
            feedingData.typeClicked = 'oral';
        }
        else if (feedingData.idtpservhosp === '1') {//Enteral
            feedingData.typeClicked = 'pare';
            widget.subitemtoadd = 'enteralProd';
            if (!feedingData.isNewRegister) {
                getCurEnteralFeeding(feedingData);
            }
        }
        else if (feedingData.idtpservhosp === '2') {//LACTARIO
            feedingData.typeClicked = 'lact';

            if (!feedingData.isNewRegister) {
                
                setLactaryFeedingOnSelectedLactaryPrescription(feedingData);
                setLactaryFeedingColumnsAndPropertiesOnFieldEdited(feedingData);
                feedingData.isLactaryPrescriptionEditing = true;
                
            }

        }
        setTimeout(function () {
            filterServicesForFeedingControlPopUp(feedingData, filter, feedingData.isNewRegister);
        }, 130);

    }
    
    $scope.editEnteralFeeding = function (entFeeding) {
        var widgetPopupItemEnt = widget.container.getWidget('widgetPopupItemEnt');
        var fieldFormulation = widgetPopupItemEnt.getField('formulationId');
        widget.isSubeditting = true;
        widgetPopupItemEnt.edit();
        $scope.setEnteralFeedingInfosOnEditingEnvironment(widget, entFeeding);
        widgetPopupItemEnt.getField('automatic').isVisible = false;
        widgetPopupItemEnt.getField('period').isVisible = false;
        widgetPopupItemEnt.getField('interval').isVisible = false;
        fieldFormulation.disabled = false;
        if(widgetPopupItemEnt.currentRow.preparationId === 'F'){
            fieldFormulation.valueField = 'productId';                                            
        } else {
            fieldFormulation.valueField = 'formulationId';
        }
        fieldFormulation.value(widgetPopupItemEnt.currentRow[fieldFormulation.valueField]);
        configurationWidget(widgetPopupItemEnt, 'modeEdit');
        ScreenService.openPopup(widgetPopupItemEnt);

    };
    
    $scope.editLactaryFeeding = function (lactaryFeeding) {
        var widgetPopupItemLac = widget.container.getWidget('widgetPopupItemLac');
        
        widgetPopupItemLac.edit();
        widgetPopupItemLac.currentRow = angular.copy(lactaryFeeding);
        widgetPopupItemLac.currentRow.lactaryDate = moment(widgetPopupItemLac.currentRow.lactaryDate, "DD/MM/YYYY").format("YYYY-MM-DD"); 
        if(typeof widgetPopupItemLac.currentRow.plateNumbers === 'string'){
            widgetPopupItemLac.currentRow.plateNumbers = parseFloat(widgetPopupItemLac.currentRow.plateNumbers.split(',').join('.'));
        } else {
            widgetPopupItemLac.currentRow.plateNumbers = parseFloat(widgetPopupItemLac.currentRow.plateNumbers);
        }
        widgetPopupItemLac.currentRow.hourLactaryPlate = widgetPopupItemLac.currentRow.hourLactaryPlate.split(":").join("");
        widgetPopupItemLac.getField('hourLactaryPlate').reload();
        widgetPopupItemLac.getField('automatic').isVisible = false;
        widgetPopupItemLac.getField('period').isVisible = false;
        widgetPopupItemLac.getField('interval').isVisible = false;
        configurationWidget(widgetPopupItemLac, 'modeEdit');
        ScreenService.openPopup(widgetPopupItemLac);
        widget.lactaryFeedingBackUp = angular.copy(lactaryFeeding);
        
        setTimeout(function(){
            $scope.onChangeLactaryFeeding(widget);
        }, 200);
        
        $scope.setLactaryFeedingInfosOnEditingEnvironment(widget, lactaryFeeding);

    };
    
    $scope.setLactaryFeedingInfosOnEditingEnvironment = function(widget, lactaryFeeding){
        
        widget.container.getWidget('widgetPopupPrescricao').currentRow.curLactaryFeeding = lactaryFeeding;
        var curLactaryFeeding = widget.container.getWidget('widgetPopupPrescricao').currentRow.curLactaryFeeding;
        
        //curLactaryFeeding.lactaryDate = moment(lactaryFeeding.lactaryDate, "DD/MM/YYYY HH:mm").format("YYYY-MM-DDTHH:mm");
        curLactaryFeeding.formulationId = curLactaryFeeding.formulationId ? curLactaryFeeding.formulationId : lactaryFeeding.formulationId;
        if(typeof curLactaryFeeding.plateNumbers === 'string'){
            curLactaryFeeding.plateNumbers = Number(lactaryFeeding.plateNumbers.split(',').join('.'));
        } else {
        curLactaryFeeding.plateNumbers = Number(lactaryFeeding.plateNumbers);
        }
        curLactaryFeeding.observation = lactaryFeeding.observation;
    };
    
    $scope.setEnteralFeedingInfosOnEditingEnvironment = function (widget, enteralFeeding) {
        var widgetPopupItemEnt = widget.container.getWidget('widgetPopupItemEnt');
        widgetPopupItemEnt.currentRow = angular.copy(enteralFeeding);
        var currentRowSubItemEnt = widgetPopupItemEnt.currentRow;
        currentRowSubItemEnt.time = moment(enteralFeeding.time,"HH:mm").format("HHmm");
        currentRowSubItemEnt.enteralDate = moment(enteralFeeding.enteralDate, "DD/MM/YYYY").format("YYYY-MM-DD");
        widgetPopupItemEnt.getField('accessRoadId').value(currentRowSubItemEnt.accessRoadId);
        widgetPopupItemEnt.getField('preparationId').value(currentRowSubItemEnt.preparationId);
        var formulation = widgetPopupItemEnt.getField('formulationId');
        var fieldAccess = widgetPopupItemEnt.getField('accessRoadId');
        fieldAccess.widget.currentRow.accessRoadId = fieldAccess.value();
        var accessRoad = fieldAccess.dataSource.data.filter(function(access) {
            return access.accessRoadId === fieldAccess.value();
        })[0];
        fieldAccess.widget.currentRow.accessRoadName = accessRoad.accessRoadName;
        fieldAccess.reload();
        fieldAccess.value(accessRoad.accessRoadId);
        formulation.dataSourceFilter = [
            {
                "name": "preparationId",
                "operator": "=",
                "value": currentRowSubItemEnt.preparationId
            },
            {
                "name": "unit",
                "operator": "=",
                "value": localStorageEngine.getLocalVar('__CDFILIAL')
            }
        ];

        formulation.value(currentRowSubItemEnt.formulationId);
        formulation.readOnly = false;
    };

    function setLactaryFeedingOnSelectedLactaryPrescription(prescription) {

        var filter = [
            {
                "name": "id",
                "operator": "=",
                "value": prescription.id
            },
            {
                "name": "unit",
                "operator": "=",
                "value": prescription.unit
            },
            {
                "name": "serviceDate",
                "operator": "=",
                "value": moment(prescription.serviceDate, "DD/MM/YYYY").format("DD/MM/YYYY")
            },
            {
                "name": "sequential",
                "operator": "=",
                "value": prescription.sequential
            }
        ];

        return lactaryFeedingCrud.find(filter,false,'/LactaryFeedings').then(function (lactaryFeedings) {
                prescription.lactaryFeedings = lactaryFeedings;
            })
            .catch(function (error) {
                console.log(error);
            });

    }

    $scope.pressToChooseAll = function (widget, dataSource) {

        widget.allFieldsAreSelected = widget.allFieldsAreSelected ? false : true;

        if (widget.allFieldsAreSelected) {
            dataSource.forEach(function (field) {
                if (!field.selectedRow) {
                    field.selectedRow = true;
                    widget.selectedRows.push(field);
                }
            });
        } else {
            widget.selectedRows = [];
            dataSource.forEach(function (field) {
                field.selectedRow = false;
            });
        }
    };// 
    //
    $scope.pressToChoose = function (row, widget, dataSource) {
        var indexOfAtr = widget.selectedRows.indexOf(row);
        if(widget.showPopUpRestr){
            indexOfAtr = widget.selectedRows.findIndex(function(rowToSelect){
                return rowToSelect.cdgrupprod === row.cdgrupprod && rowToSelect.cdsubgrprod === row.cdsubgrprod;
            });
        }
        row.selectedRow = row.selectedRow !== true ? true : false;
        if (indexOfAtr > -1) {
            widget.selectedRows.splice(indexOfAtr, 1);
        } else {
            widget.selectedRows.push(row);
        }
        if(Array.isArray(dataSource) && dataSource.length === widget.selectedRows.length){
            widget.allFieldsAreSelected = true;
        }else{
            widget.allFieldsAreSelected = false;
        }
    };
    
    $scope.afterMoveRowGrid = function(widget, container, element){
        widget.currentRow = angular.copy(element);
        switch (widget.name){
            case 'WidgetUnit':
                eventAggregator.publish('afterMoveRowUnit', container);
                break;
            case 'widgetPopupHospitalization':
                $scope.pressToChooseSelect(element,widget);
                break;
            default:
                return null;
        }
    };
    
    $scope.pressToChooseSelect = function (row, widget) {
        if(!widget.selectUnique){
            var widgetPopupPrescricao = widget.container.getWidget('widgetPopupPrescricao');
            var indexOfAtr = widget.selectedRows.findIndex(function(selectedRow){
                return row.formulationId === selectedRow.formulationId && row.plateType === selectedRow.plateType;
            });
            row.selectedRow = row.selectedRow !== true ? true : false;

            if(Array.isArray(widgetPopupPrescricao.filtredFeedingsForWidget) && widgetPopupPrescricao.filtredFeedingsForWidget.length){
                var index = widgetPopupPrescricao.filtredFeedingsForWidget.findIndex(function(feeding){
                    return feeding.formulationId === row.formulationId && feeding.plateType === row.plateType;
                });
                if(index !== -1){ 
                    widgetPopupPrescricao.filtredFeedingsForWidget.splice(index, 1);   
                }
            }
            if (indexOfAtr > -1) {
                updateBkpFeedings(widgetPopupPrescricao, widget.selectedRows[indexOfAtr], true);
                widget.selectedRows.splice(indexOfAtr, 1);
            } else {
                updateBkpFeedings(widgetPopupPrescricao, row, false);
                widget.selectedRows.push(row);
            }
            if(widget.selectedRows.length === widget.elementsToSearch.length){
                widget.allFieldsAreSelected = true;
            }else{
                widget.allFieldsAreSelected = false;
            }
        } else {
            var fieldFormulation;
            switch(widget.name){
                case 'widgetPopupFormulationLact':
                    var widgetPopupItemLac = widget.container.getWidget('widgetPopupItemLac');
                    fieldFormulation = widgetPopupItemLac.getField('formulationId');
                    fieldFormulation.value(row.cdformulacao);
                    widget.container.getWidget('widgetPopupItemLac').getField('un').value(row.un);
                    widgetPopupItemLac.currentRow.cdformulacao = row.cdformulacao; 
                    widgetPopupItemLac.currentRow.formulationName = row.nmformulacao;
                    break;
                case 'widgetPopupFormulationEnt':
                    var widgetPopupItemEnt = widget.container.getWidget('widgetPopupItemEnt');
                    fieldFormulation = widgetPopupItemEnt.getField('formulationId');
                    fieldFormulation.value(row.cdformulacao);
                    widgetPopupItemEnt.currentRow.cdformulacao = row.formulationId; 
                    widgetPopupItemEnt.currentRow.formulationId = row.formulationId; 
                    widgetPopupItemEnt.currentRow.formulationName = row.formulationName;
                    break;
                case 'widgetPopupHospitalization':
                    var newHospitalizationPopup = widget.container.getWidget('newHospitalizationPopup');
                    var fieldPatient = newHospitalizationPopup.getField('id');
                    fieldPatient.value(row.id);
                    newHospitalizationPopup.currentRow.id = row.id; 
                    newHospitalizationPopup.currentRow.idnamepaci = row.idnamepaci; 
                    break;
                    
            }    
            widget.isVisible = false;
            widget.descriptionPopupSelect.search = "";
            ScreenService.closePopup(widget);
        } 
        /*
        
            var itemList = $(event.currentTarget);
            itemList.hasClass('hide-all') ? itemList.removeClass('hide-all') : itemList.addClass('hide-all');
            itemList.siblings('.fields-dietPrescription').addClass('hide-all');
        
        */
    };
    $scope.onChangeSearch = function(fieldOrWidget, textToSearch){
        var promise = ZHPromise.defer();
        setTimeout(function(){
            var text = "";
            if(textToSearch){
                text = textToSearch;
            }else{
                text = (fieldOrWidget.search === "" || fieldOrWidget.search) ? fieldOrWidget.search : "";
                text = (!text && fieldOrWidget.descriptionPopupSelect && fieldOrWidget.descriptionPopupSelect.search) ? fieldOrWidget.descriptionPopupSelect.search : text;
            }
            text = removerAcentos(text);
            var searchFromDataSourceFilter = fieldOrWidget.searchFromDataSourceFilter;
            var arrayToFilter = fieldOrWidget[fieldOrWidget.arrayToFilter];
            var propertyToFind = fieldOrWidget.propertyToFind;
            var functionToSearch = fieldOrWidget.functionToSearch;
            if(typeof text === 'string' && typeof propertyToFind === 'string' && typeof fieldOrWidget === 'object' && text !== fieldOrWidget.lastFindText) {
                    if(searchFromDataSourceFilter){
                        if(Array.isArray(fieldOrWidget.dataSourceFilter) && fieldOrWidget.dataSourceFilter.length){
                            var index = fieldOrWidget.dataSourceFilter.findIndex(function(filter){
                                return filter.name === propertyToFind;
                            });
                            if(index > -1){
                                fieldOrWidget.dataSourceFilter[index].value = '%'+text+'%';
                            } else {
                                fieldOrWidget.dataSourceFilter.push({
                                    'name': propertyToFind,
                                    'operator': "LIKE",
                                    'value': '%'+text+'%'
                                });
                            }
                            fieldOrWidget.reload().then(function(){
                                if(fieldOrWidget.widget){
                                    fieldOrWidget.widget.reload().then(function(){
                                        promise.resolve();
                                    });
                                }else{
                                    promise.resolve();
                                }
                            }, function(){
                                promise.reject();
                            });
                        } else {
                           fieldOrWidget.dataSourceFilter = [
                                {
                                    'name': propertyToFind,
                                    'operator': "LIKE",
                                    'value': '%'+text+'%'
                                }
                           ]; 
                           fieldOrWidget.reload().then(function(){
                                if(fieldOrWidget.widget){
                                    fieldOrWidget.widget.reload().then(function(){
                                        promise.resolve();
                                    });
                                }else{
                                    promise.resolve();
                                }
                            }, function(){
                                promise.reject();
                            });
                        }
                    } else {
                        if(Array.isArray(arrayToFilter)){
                            arrayToFilter = arrayToFilter.filter(function(element){
                                return removerAcentos(element[propertyToFind]).indexOf(removerAcentos(text)) > -1; 
                            });
                            promise.resolve();
                        } else {
                            promise.reject();
                        }
                    }
                    fieldOrWidget.lastFindText = text;
            } else {
                if(text !== fieldOrWidget.lastFindText){
                        switch(functionToSearch){
                            case 'searchPlate':
                                $scope.searchPlate(widget);
                                break;
                            default:
                                promise.reject();
                        }
                        promise.resolve();
                        fieldOrWidget.lastFindText = text;
                    
                }
            }
        },1000);
        return promise;
    };
    
    function updateBkpFeedings (widgetPopupPrescricao, row, statusOfRemoved){
        if(Array.isArray(widgetPopupPrescricao.bkpFeedings) && widgetPopupPrescricao.bkpFeedings.length){
            var deleteToBkp = widgetPopupPrescricao.bkpFeedings.filter(function(bkp){
                return bkp.formulationId === row.formulationId && bkp.plateType === row.plateType;
            })[0];
            if(deleteToBkp){
                deleteToBkp.remove = statusOfRemoved;
            }
        }
    }
    
    $scope.deleteControlFeeding = function(controlFeeding){
        ScreenService.confirmMessage(ScreenService.i18n("Deseja realmente excluir este controle alimentar?"), 'question',
        function(accept){
            var widgetDetailsPatient = widget.container.getWidget('WidgetDetailsPatient');
            var index = widgetDetailsPatient.localDataSourceLoaded.ControlFeedings.indexOf(controlFeeding);
            var indexCurPatient = widgetDetailsPatient.curPatient.ControlFeedings.indexOf(controlFeeding);
            var platesToRemove;
            var dataSourceName;
            var params = {
                'widgetDetailsPatient': widgetDetailsPatient,
                'prescription': widgetDetailsPatient.localDataSourceLoaded.ControlFeedings[index]
            };
            eventAggregator.publish('callDeleteOnline', params);
            var datePrescription = controlFeeding.serviceDate;
            var dateFormat = datePrescription.indexOf('/') > -1 ? datePrescription : moment(datePrescription).format('DD/MM/YYYY');
            switch(controlFeeding.idtpservhosp){
                case '0': 
                case '3':
                case '4':
                    dataSourceName = "/NormalFeedings";
                    platesToRemove = widgetDetailsPatient.curPatient.NormalFeedings.filter(function(feeding){
                        return feeding.sequential === controlFeeding.sequential && feeding.serviceDate === dateFormat;
                    });
                    break;
                case '2':
                    dataSourceName = "/LactaryFeedings";
                    platesToRemove = widgetDetailsPatient.curPatient.LactaryFeedings.filter(function(feeding){
                        return feeding.sequential === controlFeeding.sequential && feeding.serviceDate === dateFormat;
                    });
                    break;
                case '1':
                    dataSourceName = "/EnteralFeedings";
                    platesToRemove = widgetDetailsPatient.curPatient.EnteralFeedings.filter(function(feeding){
                        return feeding.sequential === controlFeeding.sequential && feeding.serviceDate === dateFormat;
                    });
                    break;
            }
            if(dataSourceName === '/NormalFeedings'){
                eventAggregator.publish('callSaveOnline',widgetDetailsPatient.localDataSourceLoaded.ControlFeedings[index]);
            }
            var plateParams = {
                'plates': platesToRemove,
                'dataSourceName': dataSourceName,
                'widgetDetailsPatient': widgetDetailsPatient
            };
            eventAggregator.publish('callDeletePlatesOnline', plateParams);
           // controlFeedingCrud.removeDataset(platesToRemove, dataSourceName);
            widgetDetailsPatient.localDataSourceLoaded.ControlFeedings.splice(index,1);
            widgetDetailsPatient.curPatient.ControlFeedings.splice(indexCurPatient,1);
            
        });
    };
    
    $scope.deleteLactaryFeeding = function(lactaryFeeding){
        ScreenService.confirmMessage(ScreenService.i18n("Deseja realmente excluir este prato da prescrição?"), 'question', 
        function(accept){
            var widgetPopupPrescricao = widget.container.getWidget('widgetPopupPrescricao');  
            var widgetDetailsPatient = widget.container.getWidget('WidgetDetailsPatient');  
            var index = widgetPopupPrescricao.currentRow.lactaryFeedings.indexOf(lactaryFeeding);
            var funcIndex;
            var getIndexOnline = function(feeding){
                return feeding.unique === lactaryFeeding.unique && feeding.sequential === lactaryFeeding.sequential && feeding.id ===lactaryFeeding.id;
            };
            var getIndexOffline = function (feeding){
                return feeding.key === lactaryFeeding.key && feeding.id === lactaryFeeding.id;  
            };
            funcIndex = getLocalVar('__ONLINE') ? getIndexOnline : getIndexOffline;

            var indexLactaryFeedings = widgetDetailsPatient.localDataSourceLoaded.LactaryFeedings.findIndex(funcIndex);
            var indexPatientLactaryFeedings = widgetDetailsPatient.curPatient.LactaryFeedings.findIndex(funcIndex);
            // lactaryFeedingCrud.removeRow(widgetDetailsPatient.localDataSourceLoaded.LactaryFeedings[indexLactaryFeedings], "/LactaryFeedings");
            var plateParams = {
                'plates': widgetDetailsPatient.curPatient.LactaryFeedings[indexPatientLactaryFeedings],
                'dataSourceName': "/LactaryFeedings",
                'widgetDetailsPatient': widgetDetailsPatient
            };
            eventAggregator.publish('callDeletePlatesOnline', plateParams);
            widgetPopupPrescricao.currentRow.lactaryFeedings.splice(index,1);
            widgetDetailsPatient.localDataSourceLoaded.LactaryFeedings.splice(indexLactaryFeedings,1);
            widgetDetailsPatient.curPatient.LactaryFeedings.splice(indexPatientLactaryFeedings,1);
        });
    };
    $scope.deleteEnteralFeeding = function(enteralFeeding){
        ScreenService.confirmMessage(ScreenService.i18n("Deseja realmente excluir este prato da prescrição?"), 'question', 
        function(accept){
            var widgetPopupPrescricao = widget.container.getWidget('widgetPopupPrescricao');  
            var widgetDetailsPatient = widget.container.getWidget('WidgetDetailsPatient');  
            var index = widgetPopupPrescricao.currentRow.feedings.indexOf(enteralFeeding);
            var indexEnteralFeedings = widgetDetailsPatient.localDataSourceLoaded.EnteralFeedings.findIndex(function(feeding){
                return feeding.key === enteralFeeding.key;
            });
            var indexPatientEnteralFeedings = widgetDetailsPatient.curPatient.EnteralFeedings.findIndex(function(feeding){
                return feeding.key === enteralFeeding.key;
            });
            // enteralFeedingCrud.removeRow(widgetDetailsPatient.localDataSourceLoaded.EnteralFeedings[indexEnteralFeedings], "/EnteralFeedings");
            var plateParams = {
                'plates': widgetDetailsPatient.curPatient.EnteralFeedings[indexPatientEnteralFeedings],
                'dataSourceName': "/EnteralFeedings",
                'widgetDetailsPatient': widgetDetailsPatient
            };
            eventAggregator.publish('callDeletePlatesOnline', plateParams);
            widgetPopupPrescricao.currentRow.feedings.splice(index,1);
            widgetDetailsPatient.localDataSourceLoaded.EnteralFeedings.splice(indexEnteralFeedings,1);
            widgetDetailsPatient.curPatient.EnteralFeedings.splice(indexPatientEnteralFeedings,1);
        });
    };
    $scope.copiedPrescriptionDiet = function(){
        var widgetPopupPrescricao = widget.container.getWidget('widgetPopupPrescricao');
        if(widgetPopupPrescricao.getField('serviceId').value()){
            ScreenService.confirmMessage(ScreenService.i18n("Ao copiar informações da prescrição dietética, a configuração atual pode sofrer alterações. Confirmar operação?"), 'question',
            function(accept){
                var widgetDetailsPatient = widget.container.getWidget('WidgetDetailsPatient');
                var prescriptions = widgetDetailsPatient.curPatient.Prescriptions;
                if(Array.isArray(prescriptions) && prescriptions.length){
                    var service = widgetPopupPrescricao.currentRow.idformdiet;
                    var exists = false; 
                    var diets = widgetDetailsPatient.localDataSourceLoaded.Diets;
                    var dietsServices = widgetDetailsPatient.localDataSourceLoaded.DietsServices;
                    for(var i=0; i<prescriptions.length; i++){
                        var selectedDiet = diets.filter(function(diet){
                            return diet.dietId === prescriptions[i].dietId;
                        })[0];
                        if(Array.isArray(service) && service.length){
                            var index = service.indexOf(selectedDiet.service);
                            if(index > -1){
                                var foundedDiet = dietsServices.filter(function(dietService){
                                    return dietService.serviceId === widgetPopupPrescricao.getField('serviceId').value() && dietService.service === selectedDiet.service && dietService.dietId === selectedDiet.dietId;
                                });
                                if(Array.isArray(foundedDiet) && foundedDiet.length){
                                    
                                    exists = true;
                                    widgetPopupPrescricao.getField('dietId').value(selectedDiet.dietId);
                                    eventAggregator.publish('updateDietFromPrescriptionDiet', widgetPopupPrescricao);
                                    break;
                                }
                            }
                        }else{
                            if(selectedDiet.service === service){
                                var foundedDietService = dietsServices.filter(function(dietService){
                                    return dietService.serviceId === widgetPopupPrescricao.getField('serviceId').value() && dietService.service === selectedDiet.service;
                                });
                                if(Array.isArray(foundedDietService) && foundedDietService.length){
                                    exists = true;
                                    widgetPopupPrescricao.getField('dietId').value(selectedDiet.dietId);
                                    eventAggregator.publish('updateDietFromPrescriptionDiet', widgetPopupPrescricao);
                                    break;
                                }
                            }
                        }
                    }
                    if(exists){
                        ScreenService.showMessage(ScreenService.i18n("Prescrição dietética copiada com sucesso!"), 'success'); 
                    }else{
                        ScreenService.showMessage(ScreenService.i18n("Não foi encontrada prescrição dietética compatível com o tipo de prescrição que está sendo inserida!"));
                    }
                    
                }else{
                    ScreenService.showMessage(ScreenService.i18n("Não há prescrições dietéticas para o paciente atual!"));
                }
            }); 
        } else{
            ScreenService.showMessage(ScreenService.i18n("Selecione um serviço para realizar a cópia de Prescrição dietética!"));
        }
    };

    $scope.startEditing = function (otherData, model, field) {
        if(field.name == 'foodIntoleranceGroup'){
            $scope.startEdditingFoodGroup(field);
        } else{
            $scope.editorOptions = {
                toolbar: [
                    ['edit', ['undo', 'redo']],
                    ['style', ['bold', 'italic', 'underline', 'superscript', 'subscript', 'strikethrough', 'clear']],
                    ['textsize', ['fontsize']],
                    ['fontclr', ['color']]
                ]
            };
            widget.fieldEdited = {};
            widget.fieldEdited.currentEditedModel = model;
            widget.fieldEdited.isEditting = true;
            widget.fieldEdited.type = field.type;
            widget.fieldEdited.currentEditedLabel = field.label;
            widget.fieldEdited.currentEditedName = field.name;
            widget.fieldEdited.otherData = otherData;
            widget.fieldEdited.__is_new = false;
            widget.fieldEdited.serviceDate = moment().format('DD/MM/YYYY');
            widget.edit();
            widget.container.showMenu = false;
        }
    };
    
    $scope.startEdditingFoodGroup = function(field){
        widget.popupFoodGroup = angular.copy(widget.localDataSourceLoaded.AllGroupSubgroup);
        var popupFoodGroup = widget.popupFoodGroup;
        var dataFoodGroup = widget.dataFoodGroup;
        dataFoodGroup.forEach(function(group){
            var foundedGroup = popupFoodGroup.filter(function(g){
                return g.cdsubgrprod === group.cdsubgrprod && g.cdgrupprod === group.cdgrupprod;
            });
            var selectedGroup = foundedGroup[0];
            if(selectedGroup){
                selectedGroup.selectedRow = true;
            }
            
        });
        if(dataFoodGroup && dataFoodGroup.length){
        	widget.selectedRows = angular.copy(dataFoodGroup); 
        }else{
        	widget.selectedRows = [];
        }
        var rowsSelected = popupFoodGroup.filter(function(row) {
            return row.selectedRow;
        });
        widget.allFieldsAreSelected = rowsSelected.length === popupFoodGroup.length ? true : false;
        widget.showPopUpRestr = true;
        widget.edit();
    };

    $scope.getBedsSource = function (fieldEdited) {

        var isNew = fieldEdited.__is_new;
        setSource = isNew ? 'BedsPrescriptions' : 'Beds';

        var bedsSource = widget.localDataSourceLoaded[setSource];

        if (isNew) {

            if (!$scope.copyLoadedBedsSource || $scope.loadedBedsSource != bedsSource) {
                $scope.loadedBedsSource = bedsSource;
                $scope.copyLoadedBedsSource = angular.copy(bedsSource);
                $scope.copyLoadedBedsSource.push(getCurPatientHospitalizationOnBedsSource());
            }
            return $scope.copyLoadedBedsSource;

        }

        return bedsSource;
    };

    /**
     * Retorna os dados de internação do paciente ativo.
     */
    function getCurPatientHospitalizationOnBedsSource() {

        var curPatient = widget.curPatient;
        var hospitalizationsDataSourceName = "Beds";
        var hospitalizations = widget.localDataSourceLoaded[hospitalizationsDataSourceName];

        var patientsHopitalization = hospitalizations.filter(function (hospitalization) {
            return hospitalization.bed === curPatient.bed && hospitalization.hall === curPatient.hall && hospitalization.storey === curPatient.storey;
        });

        var patientHopitalization = patientsHopitalization[0];
        var serializedHospitalization = [];

        if (patientHopitalization) {
            serializedHospitalization = {
                "unit": patientHopitalization.unit,
                "storeyName": patientHopitalization.storeyName,
                "hallName": patientHopitalization.hallName,
                "bedName": patientHopitalization.bedName,
                "bed": patientHopitalization.bed
            };
        }
        return serializedHospitalization;

    }

    $scope.checkAval = function (appointMentDate) {
        var currAppDate = angular.copy(appointMentDate);
        var dateSaved = getLocalVar('__appointment');
        currAppDate = moment(currAppDate, 'DD/MM/YYYY HH:mm:ss').format('DD/MM/YYYY HH:mm');
        return (currAppDate === dateSaved || currAppDate === moment(dateSaved, 'YYYY-MM-DDTHH:mm').format('DD/MM/YYYY HH:mm') || currAppDate === moment(dateSaved, 'DD/MM/YYYY HH:mm:ss').format('DD/MM/YYYY HH:mm'));
    };
    
    $scope.showElementsHidden = function (event) {
        var itemList = $(event.currentTarget);
        itemList.hasClass('hide-all') ? itemList.removeClass('hide-all') : itemList.addClass('hide-all');
        itemList.siblings('.fields-dietPrescription').addClass('hide-all');
    };
    $scope.getNutritionalName = function (nutritionalStatusId) {
        var nutritionalStatus = widget.localDataSourceLoaded.NutritionalStatus;
        var retorno = false;
        nutritionalStatus.forEach(function (nut) {
            if (nutritionalStatusId === nut.nutritionalStatusId) {
                retorno = nut.nutritionalStatusName;
            }
        });
        return retorno;
    };

    $scope.setLocationData = function (bedId, prescription) {
        widget.localDataSourceLoaded.BedsPrescriptions.forEach(function (beditem) {
            if (beditem.bed === bedId) {
                prescription.unit = beditem.unit;
                prescription.hallName = beditem.hallName;
                prescription.storeyName = beditem.storeyName;
            }
        });
    };

    $scope.startEditingPrescription = function (prescription) {
        var indexEdited = widget.curPatient.Prescriptions.indexOf(prescription);
        var newDietPrescriptionPopup = widget.container.getWidget('dietPrescriptionPopup');
        newDietPrescriptionPopup.newRow();
        
        newDietPrescriptionPopup.currentRow = {};
        newDietPrescriptionPopup.currentRow = widget.curPatient.Prescriptions[indexEdited];
        newDietPrescriptionPopup.currentRow.isEditting = true;
        newDietPrescriptionPopup.currentRow.changed = true;
        newDietPrescriptionPopup.label = ScreenService.i18n('Editar Prescrição Dietética');
        newDietPrescriptionPopup.getField('date').readOnly = true;
        newDietPrescriptionPopup.getField('dietId').readOnly = true;
        widget.prescriptionBackup = angular.copy(widget.curPatient.Prescriptions[indexEdited]);
        reloadAllFields(newDietPrescriptionPopup);
        ScreenService.openPopup(newDietPrescriptionPopup);
    };

    $scope.createNewPrescription = function () {
        
        var newDietPrescriptionPopup = widget.container.getWidget('dietPrescriptionPopup');
        newDietPrescriptionPopup.newRow();
        widget.curEditing = 'Prescriptions';
        newDietPrescriptionPopup.label = ScreenService.i18n('Nova Prescrição Dietética');

        newDietPrescriptionPopup.currentRow.isEditting = false;
        newDietPrescriptionPopup.currentRow.id = widget.curPatient.id;
        newDietPrescriptionPopup.currentRow.unit = widget.curPatient.unit;
        newDietPrescriptionPopup.currentRow.date = moment().format("YYYY-MM-DDTHH:mm:ss");
        newDietPrescriptionPopup.currentRow.changed = true;
        newDietPrescriptionPopup.currentRow.__is_new = true;
        newDietPrescriptionPopup.currentRow.originalIsNewStatus = true;
        
        var curPatientBed = $scope.getCurPatientBedName();
        newDietPrescriptionPopup.currentRow.bed         = curPatientBed.bed;
        newDietPrescriptionPopup.currentRow.bedName     = curPatientBed.bedName;
        
        newDietPrescriptionPopup.currentRow.hall        = curPatientBed.hall;
        newDietPrescriptionPopup.currentRow.hallName    = curPatientBed.hallName;
        
        newDietPrescriptionPopup.currentRow.storey      = curPatientBed.storey;
        newDietPrescriptionPopup.currentRow.storeyName  = curPatientBed.storeyName;
        
        newDietPrescriptionPopup.getField('date').readOnly = false;
        newDietPrescriptionPopup.getField('dietId').readOnly = false;
        reloadAllFields(newDietPrescriptionPopup);
        ScreenService.openPopup(newDietPrescriptionPopup);
    };

    $scope.getCurPatientHallName = function () {

        var curPatientInfos = widget.curPatient;
        var localDataSourceLoaded = widget.localDataSourceLoaded;

        var curPatientHall = localDataSourceLoaded.Halls.filter(function (hall) {
            return hall.hall === curPatientInfos.hall;
        });

        return curPatientHall[0];

    };

    $scope.getCurPatientStoreyName = function () {

        var curPatientInfos = widget.curPatient;
        var localDataSourceLoaded = widget.localDataSourceLoaded;

        var curPatientStorey = localDataSourceLoaded.Storeys.filter(function (storey) {
            return storey.storey === curPatientInfos.storey;
        });

        return curPatientStorey[0];

    };

    $scope.getCurPatientBedName = function () {

        var curPatientInfos = widget.curPatient;
        var localDataSourceLoaded = widget.localDataSourceLoaded;

        var curPatientBed = localDataSourceLoaded.Beds.filter(function (bed) {
            return bed.bed === curPatientInfos.bed;
        });

        return curPatientBed[0];

    };

    $scope.startEditingAppointment = function (appointment) {
        var indexEdited = widget.curPatient.Appointments.indexOf(appointment);
        var openConsultaPopup = widget.container.getWidget('appointmentsPopup');

        openConsultaPopup.currentRow = widget.curPatient.Appointments[indexEdited];
        openConsultaPopup.currentRow.height = parseFloat(openConsultaPopup.currentRow.height);
        openConsultaPopup.currentRow.weight = parseFloat(openConsultaPopup.currentRow.weight);
        openConsultaPopup.currentRow.isEditting = true;
        openConsultaPopup.currentRow.__is_new = false;
        openConsultaPopup.label = ScreenService.i18n('Editar Consulta');
        widget.appointmentBackup = angular.copy(widget.curPatient.Appointments[indexEdited]);
        openConsultaPopup.edit();
        ScreenService.openPopup(openConsultaPopup);
        reloadAllFields(openConsultaPopup);

    };

    $scope.createNewApointment = function () {
        widget.curEditing = 'Appointments';
        widget.subitemtoadd = '';
        widget.onAddAppointment = true;
        var openConsultaPopup = widget.container.getWidget('appointmentsPopup');
        openConsultaPopup.newRow();
        openConsultaPopup.label = 'Nova Consulta';
        //changed push for unshift
        openConsultaPopup.currentRow.nutritionistName = getLocalVar('__NMOPERADOR');
        openConsultaPopup.currentRow.nutritionistId = getLocalVar('__CDOPERADOR');
        openConsultaPopup.currentRow.isEditting = false;
        openConsultaPopup.currentRow.id = widget.curPatient.id;
        openConsultaPopup.needToSave = true;
        openConsultaPopup.currentRow.changed = true;
        openConsultaPopup.currentRow.__is_new = true;
        openConsultaPopup.currentRow.date = moment().format('YYYY-MM-DDTHH:mm:ss');
        openConsultaPopup.actions[0].label = ScreenService.i18n('Cancelar');
        reloadAllFields(openConsultaPopup);
        ScreenService.openPopup(openConsultaPopup);
        openConsultaPopup.edit();
    };
    
    eventAggregator.subscribe('changeSelectedExam', function(widget) {
        changeSelectedExam(widget);
    });
    function changeSelectedExam(widget){
        var newExamId = widget.currentRow.examinationId;
        var newAvaliation = widget.currentRow;
        var exams = widget.getParent().localDataSourceLoaded.Examinations;
        var curExam;
        if(exams){
            exams.forEach(function (exam) {
                if (exam.examinationId === newExamId) {
                    curExam = exam;
                }
            });
    
            if (curExam) {
                newAvaliation.examinationId = curExam.examinationId;
                newAvaliation.examinationUnit = curExam.examinationUnit;
                newAvaliation.examinationName = curExam.examinationName;
                newAvaliation.examinationFormula = curExam.examinationFormula;
            }
            newAvaliation.age = widget.getParent().curPatient.age;
            newAvaliation.gender = widget.getParent().curPatient.gender;
            defineClassification(widget);
            newAvaliation.examinationId = newExamId;
        }
    }
    function defineClassification(widget){
        if(widget.currentRow.examinationValue){
            widget.currentRow.examinationStatus = setClassification(widget.currentRow);
            widget.getField('examinationStatus').setValue(ScreenService.i18n(widget.currentRow.examinationStatus));
        }else{
            widget.currentRow.examinationStatus = '';
        }
    }

    $scope.startEditingBioEval = function (bioAvaliation) {
        var bioEvalPopup = widget.container.getWidget('bioEvalPopup');
        var bioAval = angular.copy(bioAvaliation);
        bioEvalPopup.showAddEditBioExam = true;
        bioEvalPopup.isSubItemBeingAdded = false;
        bioEvalPopup.currentRow = bioAval;
        bioEvalPopup.currentRow.key = bioAval.key;
        bioEvalPopup.currentRow.oldExamination = bioEvalPopup.currentRow.examinationId;
        bioEvalPopup.currentRow.examinationValue = parseFloat(bioEvalPopup.currentRow.examinationValue);
        bioEvalPopup.currentRow.examinationDate = moment(bioEvalPopup.currentRow.examinationDate, 'DD/MM/YYYY HH:mm:ss').format('YYYY-MM-DDTHH:mm:ss');
        bioEvalPopup.currentRow.priority = parseFloat(bioEvalPopup.currentRow.priority);
        bioEvalPopup.currentRow.pregnancyWeek = parseFloat(bioEvalPopup.currentRow.pregnancyWeek);
        bioEvalPopup.getField('pregnancyWeek').readOnly = bioEvalPopup.currentRow.pregnancyId === 'S' ? false : true;
        bioEvalPopup.getField('examinationDate').readOnly = true;
        bioEvalPopup.currentRow.examinationStatus = setClassification(bioEvalPopup.currentRow);
        eventAggregator.publish("disablePregnancyWeeks", bioEvalPopup);
        reloadAllFields(bioEvalPopup);
        ScreenService.openPopup(bioEvalPopup);
        bioEvalPopup.edit();
    };
    
    $scope.openPopupFieldAutoComplete = function(field){
        if(!field.disabled){
            var widget = field.widget;
            var widgetPopup;
            switch(widget.name){
                case "widgetPopupItemEnt":
                    widgetPopup = field.widget.container.getWidget('widgetPopupFormulationEnt');
                    if(Array.isArray(widgetPopup.dataSourceFilter)){
                        widgetPopup.dataSourceFilter.forEach(function(filter){
                            if(widget.currentRow[filter.name]){
                                filter.value = widget.currentRow[filter.name];
                            }
                        });
                    }
                    break;
                case "widgetPopupItemLac":
                    widgetPopup = field.widget.container.getWidget('widgetPopupFormulationLact');
                    break;
                case "newHospitalizationPopup":
                    widgetPopup = field.widget.container.getWidget('widgetPopupHospitalization');
                    break;
            }
            if(widgetPopup){
                if(widgetPopup.propertyToFind && Array.isArray(widgetPopup.dataSourceFilter)){
                    widgetPopup.dataSourceFilter.forEach(function(filter){
                        if(filter.name === widgetPopup.propertyToFind){
                            if(filter.operator==="LIKE" || filter.operator==="LIKE_I"){
                                filter.value = "%%";
                            } else {
                                filter.value = "";
                            }
                        }
                    });
                }
                var dataSourceName = widgetPopup.descriptionPopupSelect.dataSourceName;
                if(field.dataSourceFilter && field.dataSourceFilter.length){
                    reloadFilterField(widget, [field.name], widget.currentRow).then(function(){
                        // widgetPopup.elementsToSearch = angular.copy(field.dataSource.data);
                        widgetPopup[dataSourceName] = field.dataSource.data;
                        widgetPopup.descriptionPopupSelect.titleFieldSearch = ScreenService.i18n(widgetPopup.descriptionPopupSelect.titleFieldSearch);
                        widgetPopup.reload().then(function(){
                            ScreenService.openPopup(widgetPopup);
                            widgetPopup.isVisible = true;
                        });
                    });
                }else{
                    // widgetPopup.elementsToSearch = angular.copy(field.dataSource.data);
                    // widgetPopup[dataSourceName] = angular.copy(field.dataSource.data);
                    widgetPopup.descriptionPopupSelect.titleFieldSearch = ScreenService.i18n(widgetPopup.descriptionPopupSelect.titleFieldSearch);
                    widgetPopup.reload().then(function(){
                        ScreenService.openPopup(widgetPopup);
                        widgetPopup.isVisible = true;
                    });
                    // widgetPopup.dataSource.data = [];
                }
            }
        }
    };
    function clearValueField(field){
        if(Array.isArray(field.dependentFields) && field.dependentFields.length){
            field.dependentFields.forEach(function(nameDependentField){
                var currentField = field.widget.getField(nameDependentField);
                currentField.value("");
                field.widget.currentRow[currentField.valueField] = "";
                field.widget.currentRow[currentField.descriptionField] = "";
                if(Array.isArray(currentField.dependentFields) && currentField.dependentFields.length){
                    clearValueField(currentField);
                }
            });
        }
    }
    
    $scope.clearField = function(field){
        if(!field.isReadOnly() || !field.disabled){
            clearValueField(field);
            field.value("");
            field.widget.currentRow[field.valueField] = "";
            field.widget.currentRow[field.descriptionField] = "";
        }
    };
    
    $scope.addNewAvaliation = function(widget, avaliationName){
        if(widget.isValid()){
            var appointment = [];
            appointment.__is_new = true;
            appointment.date = widget.container.getWidget('appointmentsPopup').currentRow.date;
            widget.allFieldsAreSelected = false;
            widget.getParent().allFieldsAreSelected = false;
            var existBioAval = false;
            existsAval(widget.getParent(), appointment);
            if(avaliationName === 'bioq'){
                existBioAval = widget.getParent().existsAval;
            }
            var parameters = [widget.getParent(), avaliationName, existBioAval];
            eventAggregator.publish("AppointmentSave", parameters);
        }
    };
    function openAntrList(widget){
        var antrList = widget.container.getWidget('antrList');
        antrList.edit();
        widget.subitemtoadd = 'avaliationantr';
        var filter = [{
                "name": "unit",
                "operator": "=",
                "value": getLocalVar('__CDFILIAL')
            },
            {
                "name": "appointmentDate",
                "operator": "=",
                "value": getLocalVar('__normalizedAppointment')
            },
            {
                "name": "id",
                "operator": "=",
                "value": getLocalVar('__patient')
            }
        ];
    
        localStorageEngine.getDataSource('/DataInfoEvaluations', filter).then(function(infos) {
            widget.infos = infos;
            getAvaliationsSaved(widget);
        });
        ScreenService.openPopup(antrList);
    }
    
    function removeDuplicates(originalArray, objKey, date) {
        var trimmedArray = [];
        var values = [];
        var value;

        for (var i = 0; i < originalArray.length; i++) {
            value = originalArray[i][objKey];

            if (values.indexOf(value) === -1) {
                var copyArray = angular.copy(originalArray[i]);
                copyArray.method = copyArray.methodEvaluationId;
                copyArray.date = date;
                copyArray.unit = getLocalVar('__CDFILIAL');
                trimmedArray.push(copyArray);
                values.push(value);
            }
        }
        return trimmedArray;
    }

    eventAggregator.subscribe('avaliationsSaved', function(widget) {
        getAvaliationsSaved(widget);
    });
    eventAggregator.subscribe('reloadAllFields',function(widget){
        reloadAllFields(widget);
    });
    
    function getAvaliationsSaved(widget){
        var patientWidget = widget.container.getWidget('WidgetDetailsPatient');
        var age = patientWidget.curPatient.yearAndMonth[0];
        var date;
        if(!widget.onAddAppointment){
            date = moment(getLocalVar('__appointment'), 'YYYY-MM-DDTHH:mm:ss').format('DD/MM/YYYY HH:mm:ss');
        }else{
            date = moment(getLocalVar('__appointment'), 'DD/MM/YYYY HH:mm:ss').format('DD/MM/YYYY HH:mm:ss');
        }
        var filterPatient = [{
                name: "dataIni",
                operator: "<=",
                value: age
            },
            {
                name: "dataFim",
                operator: ">=",
                value: age
            },
            {
                name: "methodEvaluationId",
                operator: "NOT_IN",
                value: ['30', '31', '32', '33', '34', '35', '36']
            }
        ];

        localStorageEngine.getDataSource('/AntroMethods', filterPatient).then(function(methods) {

            methods = removeDuplicates(methods, 'name', date);
            widget.antrLoadedFields = methods;
            if(patientWidget.infos){
                widget.selectedRows = selecionados(methods, patientWidget.infos);
                if(widget.selectedRows.length === widget.antrLoadedFields.length){
                    widget.allFieldsAreSelected = true;
                }
            }
        });
    }
    
    function selecionados(avaible, selected) {
        var selectedRows = [];
        avaible.forEach(function(disponivel){
            var founded = selected.filter(function(selecionado){
                return selecionado.methodEvaluationId === disponivel.methodEvaluationId;
            })[0];
            if(founded){
                disponivel.selectedRow = true;
                selectedRows.push(disponivel);
            }
        });
        return selectedRows;
    }

    $scope.addAvaliation = function (widget, appointment, avaliationName) {

        localStorageEngine.setLocalVar('__appointment', appointment.date);
        localStorageEngine.setLocalVar('__appointmentdata', appointment);
        localStorageEngine.setLocalVar('__normalizedAppointment', appointment.normalizedDate);
        widget.view();
        widget.allFieldsAreSelected = false;
        existsAval(widget, appointment);

        if (avaliationName === 'bioq') {
            widget.subitemtoadd = 'avaliationbioq';
            if(widget.existsAval){
                var bioExamList = widget.container.getWidget('bioqList');
                bioExamList.edit();
                ScreenService.openPopup(bioExamList);
            }else{
                eventAggregator.publish("openBioEvalPopup", widget);
            }
        } else if (avaliationName === 'antr') {
            widget.onAddAppointment = false;
            openAntrList(widget);
        }
        else {
            console.log(avaliationName);
        }
    };
    
    function existsAval(widget, appointment){
        var dateSaved = angular.copy(appointment.date);
        dateSaved = moment(appointment.date, 'YYYY-MM-DDTHH:mm:ss').format('DD/MM/YYYY HH:mm:ss');
        var exists = false;
        var bioEval = widget.curPatient.BioEvaluations;
        if(!bioEval){
            return false;
        }
        
        for(var i=0; i < bioEval.length; i++){
            if(bioEval[i].appointmentDate === dateSaved){
                exists = true;
                break;
            }
        }
        widget.existsAval = exists;
    }

    $scope.onChangeFoodControlService = function (serviceName, widget, prescription) {
        widget.redraw();
        widget.fieldEdited.selectedDiet = {};
        widget.fieldEdited.dietId = "";
        widget.fieldEdited.dietTypeName = "";
        widget.filtredFeedingForWidget = [];
        widget.filtredFeedingsForWidget = [];
        var prescriptionInfos;
        if (!Boolean(widget.fieldEdited) && Util.isEmptyOrBlank(widget.fieldEdited)) {
            prescriptionInfos = prescription;
        } else {
            prescriptionInfos = widget.currentRow ? widget.currentRow : prescription;
        }

        var filter = [
            {
                "name": "unit",
                "operator": "=",
                "value": localStorageEngine.getLocalVar('__CDFILIAL')
            },
            {
                "name": "serviceId",
                "operator": "=",
                "value": prescriptionInfos.cdservfili
            }
        ];

        if (prescriptionInfos.idtpservhosp === "0") {
            filter.push({
                "name": "service",
                "operator": "IN",
                "value": ['0', '3', '4']
            });
        } else {
            filter.push({
                "name": "service",
                "operator": "=",
                "value": prescriptionInfos.idformdiet
            });
        }

        localStorageEngine.getDataSource('/DietsServices', filter).then(function (filteredDiets) {
            widget.filteredDietsFeeding = filteredDiets;
            widget.filteredServicesFeeding.forEach(function (service) {
                if (serviceName === service.serviceId) {
                    prescriptionInfos.serviceName = service.serviceName;
                }
            });
        });
    };

    $scope.openPopupSelectPlates = function (widget){
        var widgetPopupPlates = widget.container.getWidget('widgetPopupPlates');
        widgetPopupPlates.showPopupPlates = true;
        widgetPopupPlates.isVisible = true;
        widgetPopupPlates.descriptionPopupSelect.titleFieldSearch = ScreenService.i18n(widgetPopupPlates.descriptionPopupSelect.titleFieldSearch);
        widgetPopupPlates.openPopup = true;
        eventAggregator.publish('loadDataPlatesByUnit', widgetPopupPlates);
    };
    
    
    $scope.onChangeLactaryFeeding = function (widget) {
            
        var curLactaryFeeding = widget.container.getWidget('widgetPopupPrescricao').currentRow.curLactaryFeeding;
        var cdformulacao = curLactaryFeeding.formulationId;
        
        var filter = [{
            "name": "cdformulacao",
            "operator": "=",
            "value": cdformulacao
        }];
        localStorageEngine.getDataSource('/LactaryPlates', filter).then(function (plate) {
            widget.container.getWidget('widgetPopupItemLac').currentRow.un = plate[0].un;
            widget.container.getWidget('widgetPopupItemLac').getField('formulationId').dataSource.data = [{
                "cdformulacao": plate.cdformulacao,
                "nmformulacao": plate.nmformulacao
            }];
            widget.container.getWidget('widgetPopupItemLac').getField('formulationId').reload();
        });
    };

    $scope.filterFeedingPatientDate = function (widget) {
        var curDate = widget.fieldEdited.serviceDate;
        var dateEdited = moment(curDate, 'DD/MM/YYYY').format('DD/MM/YYYY');
        var filtredFeedingsForWidget = [];

        curDate = dateEdited === "Invalid date" ? curDate : dateEdited;

        widget.localDataSourceLoaded.NormalFeedings.forEach(function (feed) {
            if (feed.serviceDate === curDate && feed.id === widget.curPatient.id) {
                filtredFeedingsForWidget.push(feed);
            }
        });
        return filtredFeedingsForWidget;
    };

    $scope.createNewFeedingControl = function (typeOfFeeding) {
        var widgetPopup = widget.container.getWidget('widgetPopupPrescricao');
        widgetPopup.typePrescription = typeOfFeeding;
        widgetPopup.currentRow = {};
        configurationWidget(widgetPopup, "modeInsert");
        var currentRow = createNewFeeding(typeOfFeeding);
        currentRow.isNewRegister = true;
        widgetPopup.getField('dietId').readOnly = true;
        switch(typeOfFeeding){
            case 'oral':
                widgetPopup.label = 'Prescrição Oral';
                widgetPopup.getField('serviceId').dataSource.name = '/OralServices';
                widgetPopup.getField('solicitante').isVisible = true;
                break;
            case 'lact':
                widgetPopup.label = 'Prescrição Lactária';
                widgetPopup.getField('solicitante').isVisible = false;
                widgetPopup.getField('serviceId').dataSource.name = '/LactaryServices';
                break;
            case 'pare':
                widgetPopup.label = 'Prescrição Enteral / Parenteral';
                widgetPopup.getField('solicitante').isVisible = false;
                widgetPopup.getField('serviceId').dataSource.name = '/EnteralServices';
                break;
        }
        openPopupPrescription(widgetPopup, currentRow);
    };

    function openPopupPrescription (widgetPopup, currentRow){
        widgetPopup.newRow();
        var isNew = widgetPopup.currentRow.__is_new;
        widgetPopup.currentRow = currentRow;
        widgetPopup.currentRow.__is_new = isNew;
        reloadFilterField(widgetPopup, ['serviceId'], widgetPopup.currentRow);
        widgetPopup.getField('serviceId').value("");
        ScreenService.openPopup(widgetPopup);
    }
    
    function reloadFilterField (widget, fields, currentRow){
        var defer = ZHPromise.defer();
        fields.forEach(function(nameField, index){
            var field = widget.getField(nameField);
            if(nameField === "dietId"){
                var idformdiet = { //Diet
                    'oral': ['0', '3', '4'],
                    'lact': '1',
                    'pare': '2'
                };
                if(field.dataSourceFilter && field.dataSourceFilter.length){
                    field.dataSourceFilter.forEach(function(currentFilter) {
                        if (currentFilter.name === "service") {
                            if (currentRow.idtpservhosp === "0") {
                                currentFilter.operator = "IN";
                            }
                            currentFilter.value = idformdiet[currentRow.typeClicked];
            
                        } else {
                            currentFilter.value = currentRow[currentFilter.name];
                        }
                    });
                    field.value(currentRow.dietId);
                }
            
            }else{
                if(field.dataSourceFilter && field.dataSourceFilter.length && field.dataSourceFilterProperties){
                    field.dataSourceFilter.forEach(function(filter){
                        filter.value = widget.currentRow[field.dataSourceFilterProperties[filter.name]];
                    });
                }
            }
            field.reload().then(function(){
                if(index === fields.length-1){
                    defer.resolve();
                }
            });
        });
        return defer;
    }
    
    function configurationWidget (widget, mode){
        Object.keys(widget[mode]).forEach(function(currentProperty){
            var property = widget[mode][currentProperty];
            if(Array.isArray(property.fields)){
                property.fields.forEach(function(nameField){
                    widget.getField(nameField)[property.name] = property.value;
                });
            }else{
                Object.keys(property.fields).forEach(function(nameField){
                    widget.getField(nameField)[property.name] = property.fields[nameField];
                });
            }
            if(typeof property.widget === "object"){
                if(property.widget.condition){
                    if(property.widget.condition.currentRow){
                        widget[property.name] = property.widget.condition.values[widget.currentRow[property.widget.condition.propertyWidget]];
                    }else if (property.widget.condition.modeEdit){
                        widget[property.name] = property.widget.condition.values["modeEdit"];
                    }else{
                        widget[property.name] = property.widget.condition.values[widget[property.widget.condition.propertyWidget]];
                    }
                }
            }
        });
    }

    $scope.startEditingFeeding = function (feeding) {
        feeding.isNewRegister = false;
        var widgetPopup = widget.container.getWidget('widgetPopupPrescricao');
        var fieldSolicitante = widgetPopup.getField('solicitante');
        var typeOfFeeding = "";
        switch(feeding.idtpservhosp){
            case "0":
            case "3":
            case "4":
                typeOfFeeding = "oral";
                fieldSolicitante.isVisible = true;
                break;
            case "1":
                typeOfFeeding = "pare";
                fieldSolicitante.isVisible = false;
                break;
            case "2":
                typeOfFeeding = "lact";
                fieldSolicitante.isVisible = false;
                break;
                
        }
        widgetPopup.typePrescription = typeOfFeeding;
        configurationWidget(widgetPopup, "modeEdit");
        
        createOrEditFeedingControl(feeding);
    };
    
     this.startEditingFeedingKitchenMaid = function(service) {
        service.isNewRegister = false;
        var widgetPopup = widget.container.getWidget('widgetPopupPrescricao');
        var fieldSolicitante = widgetPopup.getField('solicitante');
        // var typeOfFeeding = "";
        // service.idtpservhosp = "4";
        // switch(service.idtpservhosp){
        //     case "0":
        //     case "3":
        //     case "4":
        // typeOfFeeding = "oral";
        fieldSolicitante.isVisible = true;
        service.cdservfili = service.serviceId;
        // service.idformdiet = ['0', '3', '4'];
        widgetPopup.typePrescription = "oral";
        configurationWidget(widgetPopup, "modeEdit");
        
        createOrEditFeedingControl(service);
    };
    
    $scope.observationForCurrentPlate = function (feeding) {
        widget.plateObservationMode = !widget.plateObservationMode;
        feeding.plateObservationMode = !feeding.plateObservationMode;
    };
    
    $scope.showRestrictionIntoleranceByPlate = function(widget, feeding){
        var restrictionAndIntolerance;
        var intolerance = [];
        var restriction = [];
        var plates;
        var msg = "";
        if(widget.name === 'widgetPopupPrescricao'){
            restrictionAndIntolerance = widget.currentRow.PatientIntoRestriction.filter(function(restIntolerance){
                return restIntolerance.CDPRATO === feeding.formulationId && restIntolerance.CDTIPOPRAT === feeding.plateType;
            });
        } else if  (widget.name === 'WidgetDetailsPatient'){
            restrictionAndIntolerance = widget.curPatient.PatientIntoRestriction.filter(function(intoRestriction){
                return intoRestriction.NRSEQUSERV === feeding.sequential;
            });
            plates = widget.curPatient.NormalFeedings.filter(function(normalFeeding){
                return normalFeeding.sequential === feeding.sequential && normalFeeding.idrestconf === '1';
            });
        }
        if(Array.isArray(restrictionAndIntolerance) && restrictionAndIntolerance.length){
            restrictionAndIntolerance.forEach(function(restIntolerance){
                if(Array.isArray(restIntolerance.INTORESTRICTION) && restIntolerance.INTORESTRICTION.length){
                    restIntolerance.INTORESTRICTION.forEach(function(intoRestriction){
                        intoRestriction.CDPRATO = restIntolerance.CDPRATO;
                        intoRestriction.CDTIPOPRAT = restIntolerance.CDTIPOPRAT;
                        if(intoRestriction.IDINTREST === 'I'){
                            intolerance.push(intoRestriction);
                        } else if(intoRestriction.IDINTREST === 'R'){
                            restriction.push(intoRestriction);
                        }
                    });
                }
            });
            if(Array.isArray(intolerance) && intolerance.length){
                msg = msg.concat("<p class=titleStructError>"+ScreenService.i18n("Intolerâncias")+"</p><br />");
                
                intolerance.forEach(function(into){
                    if(widget.name === 'WidgetDetailsPatient' && Array.isArray(plates) && plates.length){
                        var currentPlate = plates.filter(function(plate){
                            return plate.plateType === into.CDTIPOPRAT && plate.formulationId === into.CDPRATO;
                        })[0];
                        if(currentPlate){
                            msg = msg.concat(currentPlate.plateTypeName + ' - ' + currentPlate.plateName + '<br />');
                        }
                    }
                    msg = msg.concat(into.NMSUBGRPROD+"<br />");
                });
            }
            if (Array.isArray(restriction) && restriction.length) {
                msg = msg.concat("<p class=titleStructError>"+ScreenService.i18n("Restrições")+"</p><br />");
                restriction.forEach(function(rest){
                    if(widget.name === 'WidgetDetailsPatient' && Array.isArray(plates) && plates.length){
                        var currentPlate = plates.filter(function(plate){
                            return plate.plateType === rest.CDTIPOPRAT && plate.formulationId === rest.CDPRATO;
                        })[0];
                        if(currentPlate){
                            msg = msg.concat(currentPlate.plateTypeName + ' - ' + currentPlate.plateName + '<br />');
                        }
                    }
                    msg = msg.concat(rest.NMDIAGNOST+"<br />");
                });
            }
            ScreenService.showMessage(msg);
        }

    };
    $scope.unSelectedPlate = function(feeding){
        var widgetPopup = widget.container.getWidget('widgetPopupPlates');
        var dataSourceName = widgetPopup.descriptionPopupSelect.dataSourceName;
        if(Array.isArray(widgetPopup[dataSourceName]) && widgetPopup[dataSourceName].length){
            var unSelectPlate = widgetPopup[dataSourceName].filter(function(plate){
                return plate.formulationId === feeding.formulationId && feeding.plateType;
            })[0];
            unSelectPlate.selectedRow = false;
        }
        var widgetPopupPrescricao = widgetPopup.container.getWidget('widgetPopupPrescricao');
        updateBkpFeedings(widgetPopupPrescricao, feeding, true);
        var index = widget.filtredFeedingsForWidget.indexOf(feeding);
        widget.filtredFeedingsForWidget.splice(index, 1);
        
    };
     $scope.searchPlate = function(widget) {
        var dataSourceName = widget.descriptionPopupSelect.dataSourceName;
        if(widget.name === 'widgetPopupPlates'){
            widget.elementsToSearch.forEach(function(element){
                if(element.selectedRow){
                    widget[dataSourceName].filter(function(item){
                        if(item.plateType === element.plateType && item.formulationId === element.formulationId){
                            item.selectedRow = true;
                        }
                    });
                }
            });
        }
        widget.elementsToSearch = angular.copy(widget[dataSourceName]);
        if (widget.descriptionPopupSelect.search) {
            widget.elementsToSearch = widget[dataSourceName].filter(function(element) {
                var elementName;
                var propertyToSearch = widget.descriptionPopupSelect.propertyToSearch;
                elementName = removerAcentos(element[propertyToSearch].toLowerCase());
                var changeSearch = removerAcentos(widget.descriptionPopupSelect.search.toLowerCase());
                return elementName.indexOf(changeSearch) !== -1;
            });
        }
        widget.elementsToBind = widget.elementsToSearch.reduce(function(prev, current) {
            var index = prev.findIndex(function(j) {
                return j.plateType === current.plateType;
            });
            if (index < 0) {
                index = prev.length;
                prev.push({
                    plateType: current.plateType,
                    lista: []
                });
            }
            prev[index].lista.push(current);
            return prev;
        }, []).map(function(item) {
            item.lista.opened = true;
            var selected = true;
            item.lista.forEach(function(it) {
                if (!it.selectedRow) {
                    selected = false;
                }
            });
            item.lista.selectedParent = selected;
            return item.lista;
        });
        
        if(widget.elementsToSearch.filter(row=>row.selectedRow).length == widget.elementsToSearch.length ){
            widget.allFieldsAreSelected = true;
        }else{
            widget.allFieldsAreSelected = false;
        }
        widget.reload();
    };

    function removerAcentos(newStringComAcento) {
        var string = newStringComAcento;
        var mapaAcentosHex = {
            a: /[\xE0-\xE6]/g,
            A: /[\xC0-\xC6]/g,
            e: /[\xE8-\xEB]/g,
            E: /[\xC8-\xCB]/g,
            i: /[\xEC-\xEF]/g,
            I: /[\xCC-\xCF]/g,
            o: /[\xF2-\xF6]/g,
            O: /[\xD2-\xD6]/g,
            u: /[\xF9-\xFC]/g,
            U: /[\xD9-\xDC]/g,
            c: /\xE7/g,
            C: /\xC7/g,
            n: /\xF1/g,
            N: /\xD1/g
        };

        for (var letra in mapaAcentosHex) {
            var expressaoRegular = mapaAcentosHex[letra];
            string = string.replace(expressaoRegular, letra);
        }

        return string;
    }
    $scope.desactivateCurrentPlate = function (feeding, currentRow) {
        if(currentRow.idformdiet !== '4' && currentRow.dietService !== '4'){
            feeding.selectedByUser = !feeding.selectedByUser;
            if(!feeding.selectedByUser){
                feeding.plateObservationMode = false;
                feeding.observation = "";
                if ($("#clear")[0]){
                    $("#clear")[0].style.right = "-50px";
                }
                    
            }else{
                if($("#clear")[0]){
                    $("#clear")[0].style.right = "-100px";
                }    
            }
        }
    };
    $scope.onEnteralPrescriptionSelectChange = function (curFieldSource, curValue) {

        if (curFieldSource === "Formulations") {

            widget.subItemData.formulationId = curValue;

            var filter = [{
                "name": "formulationId",
                "operator": "=",
                "value": curValue
            }];
            localStorageEngine.getDataSource("/Formulations", filter).then(function (filteredForms) {
                widget.subItemData.preparationName = filteredForms[0].preparationId
            });
        } else if (curFieldSource === "d") {

            widget.subItemData.preparationId = curValue;

            $scope.setPreparationFilter(widget);

            widget.subItemData.formulationId = null;
            widget.subItemData.formulationName = null;

        } else if (curFieldSource === "AccessRoads") {
            widget.subItemData.accessRoadId = curValue;
            //filtro para pegar o AcessRoadName que corresponde ao Id retornado pelo select da tela
            var acessName = widget.fieldSource.AccessRoads.filter(function(a){
                return a.accessRoadId === curValue;
            });
            // auxiliar para mostrar na tela de Editar Controle a AcessRoadName sem interferir na original
            widget.subItemData.acessRoad = acessName[0].accessRoadName;
        }
    };
    $scope.formatDateToRemoveTime = function (serviceDate) {
        return serviceDate.toString().split(" ")[0];
    };

    $scope.setPreparationFilter = function (widget) {

        var subItemData = widget.subItemData;

        var filter = [
            {
                "name": "preparationId",
                "operator": "=",
                "value": subItemData.preparationId
            }
        ];

        localStorageEngine.getDataSource("/Formulations", filter).then(function (filteredForms) {
            widget.fieldSource.Formulations = filteredForms;
        });

    };
    
    $scope.searchElementsGrid = function(widget){
        switch (widget.name){
            case 'WidgetUnit':
                searchUnit(widget);
                break;           
            default:
                return null;
        }    
    };
    
    function searchUnit(widget) {
        var search = angular.copy(widget.search);
        search = removerAcentos(search);
        widget.dataSourceFilter = [
            {
                "name": "NORMALIZEDNAME",
                "operator": "LIKE",
                "value": '%' + search + '%'
            },
            {
                "name": "CDFILIAL",
                "operator": "<>",
                "value": localStorageEngine.getLocalVar('__CDFILIAL')
            }
        ];
        widget.reload();
    }
    
    function reloadAllFields(widget) {
        widget.fields.forEach(function(field) {
            if (field.dataSource) {
                field.reload([{
                    "name": [{
                        "name": field.descriptionField,
                        "direction": "ASC"
                    }],
                    "operator": "ORDER_BY",
                    "value": ""
                }]);
            }
        });
    }
    
    $scope.checkFieldLength = function(args, field) {
        var maxLength = field.maxSize;
        var precision = field.precision;
        var modeToTrunc = field.modeToTrunc;
        var caracter = field.caracter;
        var value = document.getElementById(field.name).value;
        
        if (value) {
            if(field.validToSave && !isValidNumberMask(value, maxLength, precision)){
                var formatValue = formatarNumero(value, maxLength, precision, modeToTrunc, caracter);
                formatValue = parseFloat(formatValue);
                field.clearValue();
                field.setValue(formatValue);
                field.reload();
            }
            field.validToSave = true;
        }else{
            if(!field.validToSave){
                field.clearValue();
                field.validToSave = true;
            }else{
                field.validToSave = false;
            }
        }
    };
    
    function isValidNumberMask(value, maxLength, precision){
        var formatNumber = getPartsNumber(value, precision, maxLength);
        var number = value && typeof value.toString === 'function' ? value.toString().split(',').join('').split('.').join('') : "";
        if(number.length <= maxLength && formatNumber.length === value.length){
            return true;
        }else{
            return false;
        }
    }
    
    var getPartsNumber = function(number, precision, maxLength,caracter) {
        var formatNumber = '';
        number = number.split(',').join('.');
        var precisionUser = number.split('.').length > 1 ?  number.split('.')[1].length : 0; 
        if (typeof number === 'string' && typeof precision === 'number') {
            if(typeof maxLength === 'number' && (number.length-1)-precisionUser <= maxLength-precision && precisionUser <= precision && ((precisionUser !== 0) || (precisionUser === 0 && number.length <= maxLength-precision))){
                formatNumber = number;
            }else{
                var part1 = number.substring(0, number.length - precision);
                var part2 = number.substring(number.length - precision, number.length);
                if (typeof caracter === 'string') {
                    formatNumber = part1 + caracter + part2;
                } else {
                    formatNumber = part1 + '.' + part2;
                }
            }
        }
        return formatNumber;
    };
        
    var formatarNumero = function(value, maxLength, precision, modeToTrunc, caracter) {
        var arrayFromString;
        var number = value && typeof value.toString === 'function' ? value.toString().split(',').join('').split('.').join('') : "";
        if (number && typeof caracter === 'string') {
            number = value.split(caracter).join('');
        }
        if (typeof maxLength === 'number' && typeof precision === 'number') {
            if (number && number.length > maxLength) {
                do {
                    arrayFromString = number.split('');
                    if (modeToTrunc === 'END') {
                        number = number.substring(0, maxLength);
                    } else {
                        arrayFromString.shift();
                        number = arrayFromString.join('');
                    }
                } while (number.length > maxLength);
                number = getPartsNumber(number, precision, caracter);
            } else if (number.length > precision) {
                number = getPartsNumber(number, precision, caracter);
            } else if (number.length === precision){
                number = getPartsNumber(number.padStart(maxLength, '0'), precision);
            }
        } else if (typeof precision === 'number') {
            number = getPartsNumber(number, precision, caracter);
        } else if (typeof maxLength === 'number') {
            do {
                arrayFromString = number.split('');
                if (modeToTrunc === 'END') {
                    number = number.substring(0, maxLength);
                } else {
                    arrayFromString.shift();
                    number = arrayFromString.join('');
                }
            } while (number.length > maxLength);
        }
        return number;
    };
    
    $scope.selecionarTudo = function(widget){
        
        widget.allFieldsAreSelected = !widget.allFieldsAreSelected;

        if (widget.allFieldsAreSelected) {
            widget.elementsToBind.forEach(function (parent) {
                if(!parent.selectedParent){
                    parent.selectedParent = true;
                    parent.forEach(function(row){
                        if (!row.selectedRow) {
                            row.selectedRow = true;
                            widget.selectedRows.push(row);
                        }
                    });
                }
            });
        } else {
            widget.selectedRows = [];
            widget.elementsToBind.forEach(function (parent) {
                parent.selectedParent = false;
                parent.forEach(function(row){
                    row.selectedRow = false;
                });
            });
        }
    };
    
    $scope.selecionarParent = function(element, widget){
        element.selectedParent = !element.selectedParent;
        if(element.selectedParent){
            element.forEach(function(row){
                row.selectedRow = true;
                widget.selectedRows.push(row);
            });
            
            var allSelected = true;
            widget.elementsToBind.forEach(function(pai) {
                if(!pai.selectedParent){
                    allSelected = false;
                }
            });
            widget.allFieldsAreSelected = allSelected;
        }else{
            element.forEach(function(row){
                var indexOfAtr = widget.selectedRows.indexOf(row);
                if (indexOfAtr > -1) {
                    widget.selectedRows.splice(indexOfAtr, 1);
                }
                row.selectedRow = false;
            });
            widget.allFieldsAreSelected = false;
        }
    };
    
    $scope.selecionarFilho = function(row, parent, widget){
        row.selectedRow = !row.selectedRow;
        if(!row.selectedRow){
            var indexOfAtr = widget.selectedRows.indexOf(row);
            if (indexOfAtr > -1) {
                widget.selectedRows.splice(indexOfAtr, 1);
            }
            widget.allFieldsAreSelected = false;
            parent.selectedParent = false;
        }else{
            widget.selectedRows.push(row);
            var allSonSelected = true;
            parent.forEach(function(element){
                if(!element.selectedRow){
                    allSonSelected = false;
                }
            });
            parent.selectedParent = allSonSelected;
            if(allSonSelected){
                var allSelected = true;
                widget.elementsToBind.forEach(function(pai){
                    if(!pai.selectedParent){
                        allSelected = false;
                    }
                });
                widget.allFieldsAreSelected = allSelected;
            }else{
                widget.allFieldsAreSelected = false;
            }
        }
    };
}